import time
from tqdm import tqdm

# 读取数据
from utils.anno.cls.text_classification import text_classification
from utils.anno.ner.entity_extract import extract_named_entities
from utils.api.google_trans import en2cn

def write(txt):
  with open('data/cls/jd_result.csv', 'a', encoding='utf-8') as f:
    f.write(txt)

if __name__ == '__main__':
  # 多文本分类
  txts = open('data/cls/jd.csv', 'r', encoding='utf-8').read().split('\n')[1:]
  txts = [txt.split(',')[0] for txt in txts if txt != '']

  history = [
    ['一百多和三十的也看不出什么区别，包装精美，质量应该不错。', ['好评']],
    ['太失望了，根本不值这个价', ['差评']],
    ['很不错，质量挺好的，穿着蛮舒服', ['好评']],
    ['一点也不好，我买的东西拿都拿到快递员自己签收了还不给我，恶心恶心 恶心，不要脸不要脸', ['差评']],
    ['差评，客服永远不在，四天后才发的货，衣服木有以前买的好，感觉设计比例有问题，挂肩太短勒人。', ['差评']],
    ['比较简单，适合小朋友', ['好评']],
    ['还不错，挺实用的吧。。。', ['好评']],
    ['这个字帖不错，既可以 练字，还可以看看宋词', ['好评']],
    ['买的椒盐的，结果是浓浓的奶油味，很讨厌这种浓浓的食品添加剂的味道。。', ['差评']],
    ['香气跟以前的不一样，不知道是不是假的，以前的香气很好闻。', ['差评']],
  ]
  # history = []

  results = []
  for txt in tqdm(txts):
    while True:
      try:
        result = text_classification(txt, ['好评', '差评'], history)
        results.append(result)
        write(f'{txt},{result}\n')
        break
      except Exception as e:
        print(e)
        if e.http_status == 429:
          print('等待60s...')
          time.sleep(60)
          continue
        else:
          print('等待5s...')
          time.sleep(5)
        continue



import torch
import clip
from PIL import Image

device = "cuda" if torch.cuda.is_available() else "cpu"
model, preprocess = clip.load("ViT-B/32", device=device)

def cv_cls_openai(img, texts):
    image = preprocess(img).unsqueeze(0).to(device)
    text = clip.tokenize(texts).to(device)

    with torch.no_grad():
        # image_features = model.encode_image(image)
        # text_features = model.encode_text(text)
        
        logits_per_image, logits_per_text = model(image, text)
        probs = logits_per_image.softmax(dim=-1).cpu().numpy()

    # print("Label probs:", probs)  # prints: [[0.9927937  0.00421068 0.00299572]]
    return texts[probs.argmax()]

if __name__ == '__main__':
    result = cv_cls_openai(Image.open("CLIP.png"), ["a diagram", "a dog", "a cat"])
    print('result:', result)



def bio_2_json_one(anno_txt):
    ls = anno_txt.split('\n')
    text = ''
    anno = []
    now_label = ''
    for i, l in enumerate(ls):
        char, label = l.split('\t')
        text += char
        if 'B-' in label:
            start = i
            now_label = label.split('-')[1]
        if label == 'O':
            if now_label:
                anno.append([start, i, text[start:i], now_label])
                now_label = ''
                start = 0
    if now_label:
        i += 1
        anno.append([start, i, text[start:i], now_label])
    return {'text': text, 'anno': anno}


def bit_2_json(txt):
    anno_txts = txt.split('\n\n')
    annos = []
    for anno_txt in anno_txts:
        if anno_txt == '':
            continue
        anno_j = bio_2_json_one(anno_txt)
        annos.append(anno_j)
    return annos


if __name__ == '__main__':
    txt = '''你\tB-PER
是\tO
一\tO
个\tO
聪\tB-PER
明\tI-PER
的\tO
软\tB-ORG
件\tI-ORG
工\tI-ORG
程\tI-ORG
师\tI-ORG'''
    # txt = open('data/ner/weibo_ner/dev.txt', 'r', encoding='utf-8').read()
    annos = bit_2_json(txt)
    print(annos)



import openai

i = 0

def chat_openai(user):
    from ...local_config import config
    openai_key = config['openai']['key']
    if type(openai_key) == list:
        openai_keys = openai_key
    else:
        openai_keys = [openai_key]

    # Set up your API key
    openai.api_key = openai_keys[0]
    global i
    i += 1
    if i >= len(openai_keys):
        i = 0
    openai.api_key = openai_keys[i]
    # openai.api_base = "https://api.openai.com/v1"
    # openai.api_base = "http://47.89.230.109/v1"
    # openai.api_base = "https://api.tekii.cn/v1"
    openai.api_base = "https://api.aiproxy.io/v1"
    # Call the OpenAI API
    completion = openai.ChatCompletion.create(
                    model="gpt-4",
                    messages=[
                        {"role": "user", "content": f"{user}"},
                    ]
                )

    # Extract the output and parse the JSON array
    content = completion.choices[0].message.content

    return content

if __name__ == '__main__':
    content = chat_openai('123')
    print(content)
    # import flask
    # app = flask.Flask(__name__)
    # @app.route('/chat', methods=['POST'])
    # def chat_api():
    #     user = flask.request.form.get('user')
    #     return chat(user)
    # app.run(port=5000, host='0.0.0.0')



import gradio as gr
import json
import os
import numpy as np
import time
import re

from utils.anno.cls.text_classification import text_classification
from utils.anno.ner.entity_extract import extract_named_entities
from utils.anno.gen.text_generate import text_generate
from local_config import en2cn, emb, config
from utils.format.txt_2_list import txt_2_list
from utils.format.wash import wash_tel, wash_idcard, wash_q_2_b
from utils.auto_learn.cluster_text import cluster_text

os.makedirs(f'tmp/emb/', exist_ok=True)
types_md5_vector_map = {}
emb_json_path = f'tmp/emb/{config["emb"]}.json'
if os.path.exists(emb_json_path):
  types_md5_vector_map = json.load(open(emb_json_path, 'r', encoding='utf-8'))
os.makedirs('tmp/anno/', exist_ok=True)
app_t = int(time.time())

def md5(txt):
  import hashlib
  m = hashlib.md5()
  m.update(txt.encode('utf-8'))
  return m.hexdigest()

def get_txts(file):
  if type(file) == str:
    txts = file.strip().split('\n')
  else:
    txts = open(file.name, 'r', encoding='utf-8').read().strip().split('\n')
  return txts

def get_qa(txt):
  split_key = '[\t\|]'
  qa = re.compile(split_key).split(txt)
  if len(qa) < 2:
    qa.append('')
  q = qa[0]
  a = qa[1]
  return q, a

def load_example_file(file_example, md5_vector_map):
  try:
    is_emb_update = False
    train_txts = get_txts(file_example)
    for train_txt in train_txts:
      q, a = get_qa(train_txt)
      if q not in md5_vector_map or md5_vector_map[q]['a'] != a:
        is_emb_update = True
        vector = emb(q)
        md5_vector_map[q] = {
          'vector': vector,
          'q': q,
          'a': a
        }
    if is_emb_update:
      json.dump(types_md5_vector_map, open(emb_json_path, 'w', encoding='utf-8'), ensure_ascii=False, indent=2)
  except Exception as e:
    print(e)
    print('已标注文件解析失败，请一行为一条数据，输入和输出用\t分割')

def load_similar_txt(txt, md5_vector_map, sample_txts):
  vector = emb(txt)
  vec_score_arr = []
  for _txt in sample_txts:
    q, _ = get_qa(_txt)
    if q not in md5_vector_map:
      continue
    vector_info = md5_vector_map[q]
    t_vector = vector_info['vector']
    similar_score = np.dot(t_vector, vector)
    vec_score_arr.append((vector_info, similar_score))
  vec_score_arr.sort(key=lambda x: x[1], reverse=True)
  history = []
  for vec_score in vec_score_arr[:4]:
    vec_info = vec_score[0]
    history.append([vec_info['q'], vec_info['a']])
  return history
# 多线程
from concurrent.futures import ThreadPoolExecutor
executor = ThreadPoolExecutor(max_workers=30)
from tqdm import tqdm

def thread_auto_anno(out_txts, i, pbar, txt, types_txt, radio, checkbox_group, cls_prompt, ner_prompt, file_example=None):
  try:
    out_anno, txt = auto_anno(txt, types_txt, radio, checkbox_group, cls_prompt, ner_prompt, file_example=file_example)
    if radio == '无':
      out_txt = txt
    if radio == '数据生成':
      out_txt = out_anno
    else:
      out_txt = f'{txt}\t{out_anno}'
    out_txts.append([i, out_txt])
  except Exception as e:
    print('ERROR', e)
    out_txts.append([i, ''])
  pbar.update(1)
  return out_txts

def file_auto_anno(file, types_txt, radio, checkbox_group, cls_prompt, ner_prompt, file_example=None):
  sts = time.time()
  try:
    if type(file) == str:
      txts = file.strip().split('\n')
    else:
      txts = open(file.name, 'r', encoding='utf-8').read().strip().split('\n')
  except Exception as e:
    return '请输入待标注内容，其中每一行都为一句待标注原文'
  out_txts = []
  if radio == '聚类择优':
    result = cluster_text(txts, n_clusters=5)
    return '\n'.join(result)
  txts_len = len(txts)
  pbar = tqdm(total=txts_len)
  for i in range(txts_len):
    txt = txts[i]
    if radio in ['文本分类', '实体抽取']:
      txt = txt.split('\t')[0]
    # thread_auto_anno(out_txts, i, pbar, txt, types_txt, radio, checkbox_group, cls_prompt, ner_prompt, file_example=file_example)
    executor.submit(thread_auto_anno, out_txts, i, pbar, txt, types_txt, radio, checkbox_group, cls_prompt, ner_prompt, file_example=file_example)
  while len(out_txts) < txts_len:
    time.sleep(0.1)
    if time.time() - sts > 60 * 10:
      print('耗时超过10分钟，已翻译的数据已缓存，请稍后再试')
      break
  out_txts.sort(key=lambda x: x[0])
  out_txts = [f'{out_txt}' for i, out_txt in out_txts]
  return '\n'.join(out_txts)

def auto_anno(txt, types_txt, radio, checkbox_group, cls_prompt, ner_prompt, file_example=None):
  history = []
  need_trans = '翻译成中文' in checkbox_group
  need_wash_tel = '手机号脱敏' in checkbox_group
  need_wash_idcard = '身份证脱敏' in checkbox_group
  if file_example:
    types_key = f'{radio}:{types_txt}'
    if types_key not in types_md5_vector_map:
      types_md5_vector_map[types_key] = {}
    md5_vector_map = types_md5_vector_map[types_key]
    load_example_file(file_example, md5_vector_map)
    sample_txts = get_txts(file_example)
    history = load_similar_txt(txt, md5_vector_map, sample_txts)
    while len(json.dumps(history)) > 1500:
      history = history[:-1]
  if need_wash_tel:
    txt = wash_tel(txt)
  if need_wash_idcard:
    txt = wash_idcard(txt)
  if need_trans:
    # 单纯翻译 .tsv 数据集
    if radio == '无':
      cn_txt = ''
      for _txt in txt.split('\t'):
        cn = en2cn(_txt)
        cn = cn.replace('\n', ' ')
        cn_txt += cn + '\t'
      return '', cn_txt[:-1]
    else:
      txt = en2cn(txt)
  types = txt_2_list(types_txt)
  result = []
  if radio == '文本分类':
    result = text_classification(txt, types, prompt=cls_prompt, history=history)
    result = json.dumps(result, ensure_ascii=False)
  if radio == '实体抽取':
    result = extract_named_entities(txt, types, prompt=ner_prompt, history=history)
    result = json.dumps(result, ensure_ascii=False)
  if radio == '数据生成':
    result = text_generate(types, history=history)
    result = [r[0] + '\t' + json.dumps(r[1], ensure_ascii=False) for r in result]
    result = '\n'.join(result)
    txt = ''
  try:
    # 记录数据
    app_types = '、'.join(types)
    open(f'tmp/anno/{app_t}_{radio}_{app_types}.tsv', 'a', encoding='utf-8').write(f'{txt}\t{result}'.strip() + '\n')
  except:
    pass
  return result, txt

with gr.Blocks() as demo:
    demo.css = ''
    with gr.Row():
        gr.Markdown("""自动标注，大模型使用了一言千帆的api，本项目开源地址为：https://github.com/LLMLab/auto_anno""")
    with gr.Row():
        with gr.Column(variant="panel"):
            input2 = gr.Textbox(lines=3, label="输入类别", value="友好、不友好")
            cls_prompt = gr.Textbox(lines=3, label="分类提示", value='你是一个有百年经验的文本分类器，回复以下句子的分类类别，类别选项为{类别}\n{历史}输入|```{原文}```输出|', visible=False)
            ner_prompt = gr.Textbox(lines=3, label="抽取提示", value='你是一个经验丰富的命名实体抽取程序。输出标准数组json格式并且标记实体在文本中的位置\n示例输入|```联系方式：188****5678，联系地址：幸福大街20号```类型[\'手机号\', \'地址\'] 输出|[{"name": "188****5678", "type": "手机号", "start": 5, "end": 16}, {"name": "幸福大街20号", "type": "地址", "start": 5, "end": 16}]\n{历史}输入|```{原文}```类型{类别}输出|', visible=False)
            checkbox_group = gr.CheckboxGroup(["翻译成中文", "手机号脱敏", "身份证脱敏"], label="数据处理", info="")
            radio = gr.Radio(["数据生成", "聚类择优", "文本分类", "实体抽取", "无"], label="算法类型", value="文本分类")
            file_example = gr.Textbox(lines=3, label="已标注文本", value="")
            
        with gr.Column(variant="panel"):
            input1 = gr.Textbox(lines=3, label="待标注文本", value="Hello world!")
            output = gr.Textbox(label="输出结果", lines=3)
            # 输入输出
            inputs = [input1, input2, radio, checkbox_group, cls_prompt, ner_prompt, file_example]
            with gr.Row():
              btn = gr.Button("清空").style(full_width=True)
              btn2 = gr.Button("标注", visible=True, variant="primary").style(full_width=True)
              btn.click(lambda: "", [], output)
              btn2.click(file_auto_anno, inputs, output)
    with gr.Row():
        gr.Examples(examples=[
          ['前四个月我国外贸进出口同比增长 5.8%', '政治；经济；科技；文化；娱乐；民生；军事；教育；环保；其它', '文本分类', []],
          ['There is a cat trapped on the Avenue of Happiness', '地点', '实体抽取', ['翻译成中文']],
          ['联系方式：18812345678，联系地址：幸福大街20号', '手机号、地址', '实体抽取', ['手机号脱敏']],
        ], inputs=inputs)

if __name__ == '__main__':
  demo.launch(share=False)



# import os
# import sys
# sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)))

from .utils.anno.cls.text_classification import text_classification as cls
from .utils.anno.ner.entity_extract import extract_named_entities as ner
from .utils.anno.gen.text_generate import text_generate as gen
from .local_config import config



from PIL import Image
import cv2
import numpy as np
from pathlib import Path

DEBUG_ON = True

def write_image(path_name, mat):
    if DEBUG_ON:
        cv2.imwrite(path_name, mat)

file_string_output_root = "./imgOut/"

# 检测框
class Rectangle:
    def __init__(self, x, y, width, height, angle, types):
        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.angle = angle
        self.types = types

# 检测结果结构体
class DetectionResult:
    def __init__(self):
        self.rect_angles = []
        self.contours = []

# 图像预处理参数，用于分割
class PreProcessParams:
    def __init__(self):
        self.adaGray = 50
        self.minGray = 50
        self.maxGray = 255
        self.closRadius = 2
        self.openRadius = 2
        self.adaSize = 50
        self.offset = 10

        self.processBright = False
        self.processDark = False
        self.imgProcessCloseing = False
        self.imgProcessOpening = False
        self.imgProcessAda = False

Path = str

preProcessParams = PreProcessParams()
preProcessParams.adaGray = 190
preProcessParams.minGray = 220
preProcessParams.maxGray = 255
preProcessParams.closRadius = 2
preProcessParams.openRadius = 2
preProcessParams.adaSize = 81
preProcessParams.offset = -15
preProcessParams.processBright = True  # Find white objects
preProcessParams.processDark = False  # Find dark objects
preProcessParams.imgProcessCloseing = True  # Enable closing operation
preProcessParams.imgProcessOpening = True  # Enable opening operation
preProcessParams.imgProcessAda = True  # Enable adaptive thresholding

def imgPreProcess(imgGray, preProcessParams):
    imgBright = np.zeros_like(imgGray)
    imgDark = np.zeros_like(imgGray)

    if preProcessParams.processBright:
        # Thresholding for bright objects
        ret, imgBright = cv2.threshold(
            imgGray, preProcessParams.minGray, preProcessParams.maxGray, cv2.THRESH_BINARY)
        cv2.imwrite(file_string_output_root+'imgBright.png', imgBright)

        if preProcessParams.imgProcessAda:
            # Adaptive thresholding for dark objects
            imgFilt = np.zeros_like(imgGray)
            ret, imgFilt = cv2.threshold(
                imgGray, preProcessParams.adaGray, preProcessParams.maxGray, cv2.THRESH_BINARY)
            imgAda = cv2.adaptiveThreshold(imgGray, 255, cv2.ADAPTIVE_THRESH_MEAN_C,
                                  cv2.THRESH_BINARY, preProcessParams.adaSize, preProcessParams.offset)
            imgBright = (imgFilt & imgAda) | imgBright

            cv2.imwrite(file_string_output_root+'imgAda.png', imgAda)
            cv2.imwrite(file_string_output_root+'imgFilt.png', imgFilt)
            cv2.imwrite(file_string_output_root+'imgBright.png', imgBright)

        if preProcessParams.imgProcessCloseing:
            # Morphological closing for bright objects
            kernel = np.ones((preProcessParams.closRadius,
                             preProcessParams.closRadius), np.uint8)
            cv2.morphologyEx(imgBright, cv2.MORPH_CLOSE, kernel)
            cv2.imwrite(file_string_output_root+'imgBright.png', imgBright)

        if preProcessParams.imgProcessOpening:
            # Morphological opening for bright objects
            kernel = np.ones((preProcessParams.openRadius,
                             preProcessParams.openRadius), np.uint8)
            cv2.morphologyEx(imgBright, cv2.MORPH_OPEN, kernel)
            cv2.imwrite(file_string_output_root+'imgBright.png', imgBright)
        
        img = imgBright
    if preProcessParams.processDark:
        # region processDark
        # Thresholding operation to get dark regions
        cv2.threshold(imgGray, preProcessParams.minGray,
                      preProcessParams.maxGray, cv2.THRESH_BINARY_INV, imgDark)
        cv2.imwrite(file_string_output_root + "imgDark.png", imgDark)

        if preProcessParams.imgProcessAda:
            # Thresholding operation to get filtered image
            cv2.threshold(imgGray, preProcessParams.adaGray,
                          preProcessParams.maxGray, cv2.THRESH_BINARY_INV, imgFilt)
            # Adaptive thresholding operation to get AdaGrad image
            cv2.adaptiveThreshold(imgGray, 255, cv2.ADAPTIVE_THRESH_MEAN_C, cv2.THRESH_BINARY_INV,
                                  preProcessParams.adaSize, preProcessParams.offset, imgAda)
            # Combine filtered and AdaGrad images with dark regions
            imgDark = (imgFilt & imgAda) | imgDark

            cv2.imwrite(file_string_output_root + "imgAda.png", imgAda)
            cv2.imwrite(file_string_output_root + "imgFilt.png", imgFilt)
            cv2.imwrite(file_string_output_root + "imgDark.png", imgDark)

        if preProcessParams.imgProcessCloseing:
            # Get structuring element for closing operation
            e = cv2.getStructuringElement(
                cv2.MORPH_ELLIPSE, (preProcessParams.closRadius, preProcessParams.closRadius))
            # Close operation on dark regions
            cv2.morphologyEx(imgDark, cv2.MORPH_CLOSE, e)
            cv2.imwrite(file_string_output_root + "imgDark.png", imgDark)

        if preProcessParams.imgProcessOpening:
            # Get structuring element for opening operation
            e = cv2.getStructuringElement(
                cv2.MORPH_ELLIPSE, (preProcessParams.openRadius, preProcessParams.openRadius))
            # Open operation on dark regions
            cv2.morphologyEx(imgDark, cv2.MORPH_OPEN, e)
            cv2.imwrite(file_string_output_root + "imgDark.png", imgDark)

        img = imgDark
    # endregion

    return img


def traditional_detect(imgGray, preProcessParams=preProcessParams, result=DetectionResult()):

    # Convert grayscale image to BGR
    img = imgPreProcess(imgGray, preProcessParams)

    # 找轮廓
    contours = []
    hierarchy = []
    ret, thresh = cv2.threshold(
        img, 127, 255, cv2.THRESH_BINARY)  # Thresholding the image
    contours, hierarchy = cv2.findContours(
        thresh, cv2.RETR_CCOMP, cv2.CHAIN_APPROX_NONE)
    hierarchy = hierarchy[0]

    # 选出合适的轮廓
    areaThre = 400
    widthThre = 30
    heightThre = 30
    result.rect_angles = []
    for i in range(len(contours)):
        if hierarchy[i][3] == -1 and cv2.contourArea(contours[i]) > areaThre and cv2.arcLength(contours[i], True) > widthThre and cv2.arcLength(contours[i], True) > heightThre:
            rectangle = {}
            (rectangle['x'], rectangle['y']), (rectangle['width'], rectangle['height']), rectangle['angle'] = cv2.minAreaRect(
                contours[i])
            result.rect_angles.append(rectangle)
            # cv2.drawContours(imgGray, contours, i, (0, 0, 255), 8)
    result.contours = [c.tolist() for c in contours]
    # cv2.imwrite(file_string_output_root + "imgDrawContours.png", imgGray)
    return result

def main():
    svmfilestring = "./imgOut/"
    # Convert to grayscale
    imgGray = Image.open(svmfilestring + "standard.bmp").convert('L')
    result = DetectionResult()
    traditional_detect(np.array(imgGray), preProcessParams, result)
    print(result.rect_angles)


if __name__ == "__main__":
    main()



import json
import re
from tqdm import tqdm

cnt = 0
instruction_regs = [
    # 关系抽取
    # r'我将给你个输入，请根据关系列表：(.*?)，从输入中抽取出可能包含的关系三元组，并以(.*?)的形式回答。',
    # r'我希望你根据关系列表从给定的输入中抽取可能的关系三元组，并以(.*?)的格式回答，关系列表=(.*?)。',
    # r'给定的关系列表是：(.*?)\n根据关系列表抽取关系三元组，在这个句子中可能包含哪些关系三元组？请以(.*?)的格式回答。',
    # r'已知候选的关系列表：(.*?)，请你根据关系列表，从以下输入中抽取出可能存在的头实体与尾实体，并给出对应的关系三元组。请按照(.*?)的格式回答。'
    # '从给定的文本中提取出可能的实体和实体类型',
    # 实体抽取
    r'给定的实体类型列表是(.*?)\n根据实体类型列表抽取，在这个句子中可能包含哪些实体？你可以先别出实体, 再判断实体类型。请以(.*?)的格式回答。',
    r'我希望你根据实体类型列表从给定的输入中抽取可能的实体，并以(.*?)的格式回答，实体类型列表=(.*?)。',
    r'我将给你个输入，请根据实体类型列表：(.*?)，从输入中抽取出可能包含的实体，并以(.*?)的形式回答。',
    r'已知候选的实体类型列表：(.*?)，请你根据实体类型列表，从以下输入中抽取出可能存在的实体。请按照(.*?)的格式回答。',
]

out_output_reg_map = {
    "JSON字符串[{'entity':'', 'entity_type':''}, ]": r"{'entity': '(.*?)', 'entity_type': '(.*?)'}",
    '"实体是\n实体类型是\n\n"': r'实体是(.*?)\n实体类型是(.*?)\n\n',
    '"实体：实体类型\n"': r'(.*?)：(.*?)\n',
    '"(实体,实体类型)"': r'\((.*?),(.*?)\)'
}

def get_ready_key(name, type, start):
    return f'{name}-{type}-{start}'

def get_kvs_json(kvs, text):
    annos = []
    ks = set()
    for k in kvs:
        vs = kvs[k]
        if type(vs) == str:
            vs = [vs]
        for v in vs:
            if v in text:
                ks.add(k)
                annos.append([v, k, text.index(v)])
    return list(ks), annos

def llm_2_json(fn):
    ls = open(fn, 'r', encoding='utf-8').read().strip().split('\n')
    fh = open(re.sub(r'\..*', '.aa.jsonl', fn), 'w', encoding='utf-8')
    input_set = set()
    type_set = set()
    for l in tqdm(ls):
        j = json.loads(l)
        input = j['input']
        is_in = False
        for i in instruction_regs:
            results = re.findall(i, j['instruction'], re.S)
            if results:
                result = results[0]
                if result[0][0] == '[':
                    types, out_format = eval(result[0]), result[1]
                else:
                    types, out_format = eval(result[1]), result[0]
                
                annos = []
                match_times = 0
                output_reg = out_output_reg_map[out_format]
                output = j['output'].replace('输入中包含的实体是：\n', '')
                annos = re.findall(output_reg, output, re.S)
                # 因为原始数据为 姓名：NAN，说明数据错了，所以转回来
                if out_format == '"实体：实体类型\n"':
                    annos = [(item[1], item[0]) for item in annos]
                # 把没有结果的实体去掉
                annos = [item for item in annos if item[0] != 'NAN']
                # 补充实体位置
                _annos = []
                ready_keys = set()
                for anno in annos:
                    for i in range(len(input)):
                        if input[i:i+len(anno[0])] != anno[0]:
                            continue
                        # 跳过匹配过的实体，防止重复匹配
                        ready_key = get_ready_key(anno[0], anno[1], i)
                        if ready_keys.__contains__(ready_key):
                            continue
                        ready_keys.add(ready_key)
                        _annos.append([anno[0], anno[1], i])
                        break
                annos = _annos
                if annos:
                    match_times += 1
                    # print(annos)
                is_in = True
                continue
        j = {
            'types': types,
            'annos': annos,
            'input': input,
        }
        type_set.update(types)
        input_set.add(j['input'])
        fh.write(json.dumps(j, ensure_ascii=False) + '\n')
        if not annos:
            # print(j['output'])
            pass
        if not is_in:
            cnt += 1
            print(j['instruction'])
            pass
    print(len(input_set))
    print(len(type_set))

def json_2_json(fn, trans_fun, ls_fun=None):
    src = open(fn, 'r', encoding='utf-8').read().strip()
    if ls_fun:
        ls = ls_fun(src)
    elif src[0] == '[':
        ls = json.loads(src)
    else:
        ls = src.split('\n')
    out_ls = []
    out_js = []
    for l in ls:
        if type(l) == str:
            j = json.loads(l)
        else:
            j = l
        out_j = {
            'types': trans_fun['types'](j),
            'annos': trans_fun['annos'](j),
            'input': trans_fun['input'](j),    
        }
        out_ls.append(json.dumps(out_j, ensure_ascii=False))
        out_js.append(out_j)
    open(re.sub(r'\..*', '.aa.jsonl', fn), 'w', encoding='utf-8').write('\n'.join(out_ls))
    return out_js
        

if __name__ == "__main__":
    import random

    # fn = 'E:/NLP/NER/KnowLM-IE.json'
    # fn = 'E:/NLP/NER/48_ner_knowlm-ke_ner_train.json'
    # llm_2_json(fn)
    # fn = 'E:/NLP/NER/03_ner_yidu_s4k_subtask1_training_part.json'
    # json_2_json(fn, trans_fun={
    #     'types': lambda j: list(set([entity['label_type'] for entity in j['entities']])),
    #     'annos': lambda j: [[j['originalText'][entity['start_pos']:entity['end_pos']] , entity['label_type'], entity['start_pos']] for entity in j['entities']],
    #     'input': lambda j: j['originalText'],
    # })
    # fn = 'E:/NLP/NER/05_ner_medical_ner_entities.json'
    # json_2_json(fn, trans_fun={
    #     'types': lambda j: list(set([entity['label'] for entity in j['annotations']])),
    #     'annos': lambda j: [[entity['entity'] , entity['label'], entity['start_offset']] for entity in j['annotations']],
    #     'input': lambda j: j['text'],
    # })
    # fn = 'E:/NLP/NER/06_ner_MedDG_train.json'
    # types = 'Symptom Medicine Test Attribute Disease'.split(' ')
    # def get_annos(j):
    #     annos = []
    #     for type in types:
    #         if j[type]:
    #             for entity in j[type]:
    #                 if entity in j['Sentence']:
    #                     annos.append([entity, type, j['Sentence'].index(entity)])
    #     return annos
    # json_2_json(fn, trans_fun={
    #     'types': lambda j: types,
    #     'annos': lambda j: get_annos(j),
    #     'input': lambda j: j['Sentence'],
    # }, ls_fun=lambda src: sum(json.loads(src), []))
    # fn = 'E:/NLP/NER/38_ner_COTE_train.tsv'
    # types = ['评价对象']
    # json_2_json(fn, trans_fun={
    #     'types': lambda j: types,
    #     'annos': lambda j: [[j[0], '评价对象', j[1].index(j[0])]] if len(j) == 2 and j[0] in j[1] else [],
    #     'input': lambda j: j[1],
    # }, ls_fun=lambda src: [l.split('\t') for l in src.split('\n')][1:])
    # fn = 'E:/NLP/NER/44_ner_guwenee.json'
    # json_2_json(fn, trans_fun={
    #     'types': lambda j: list(set([arg['role'] for arg in sum([event['arguments'] for event in j['event_list']], [])])),
    #     'annos': lambda j: [[arg['argument'], arg['role'], j['text'].index(arg['argument'])] for arg in sum([event['arguments'] for event in j['event_list']], [])],
    #     'input': lambda j: j['text'],
    # })
    # fn = 'E:/NLP/NER/20_ner_odkg_document.json'
    # json_2_json(fn, trans_fun={
    #     'types': lambda j: get_kvs_json(json.loads(json.loads(j['业务抽取'])), j['正文'])[0],
    #     'annos': lambda j: get_kvs_json(json.loads(json.loads(j['业务抽取'])), j['正文'])[1],
    #     'input': lambda j: j['正文'],
    # })
    fn = 'E:/NLP/NER/32_ner_event_type_entity_extract_train.csv'
    types = ['涉嫌传销', '资金账户风险', '财务造假', '失联跑路', '涉嫌欺诈', '重组失败', '资产负面', '产品违规', '业绩下滑', '不能履职', '歇业停业', '公司股市异常', '信批违规', '高管负面', '提现困难', '其他', '实控人股东变更', '投诉维权', '涉嫌违法', '涉嫌非法集资', '评级调整', '交易违规']
    out_js = json_2_json(fn, trans_fun={
        'types': lambda j: [j[-2]],
        'annos': lambda j: [[j[-1], j[-2], j[1].index(j[-1])]] if j[-1] in j[1] else [],
        'input': lambda j: j[1],
    }, ls_fun=lambda src: [l[1:-1].split('","') for l in src.split('\n')])
    # types = list(set(sum([j['types'] for j in out_js], [])))
    



from setuptools import setup, find_packages


setup(
    name='auto_anno_2',
    version='0.1.6',
    packages=find_packages(),
    # 仓库地址
    url='https://github.com/LLMLab/auto_anno',
    # 作者
    author="LLMLab",
    # 作者邮箱
    author_email="541182180@qq.com",
    # 描述
    description='auto_anno_2',
    # 项目主页
    project_urls={
        "Bug Tracker": "https://github.com/LLMLab/auto_anno",
        "Documentation": "https://github.com/LLMLab/auto_anno",
        "Source Code": "https://github.com/LLMLab/auto_anno",
    },
    # 项目分类
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    # 关键字
    keywords='auto_anno_2',
    # 安装依赖
    install_requires=[
        # list your project dependencies here
        'erniebot==0.3.1'
    ],
)

# python setup.py sdist bdist_wheel # 打包
# pip install dist/auto_anno_2-*-py3-none-any.whl # 本地安装，包名调整为打包出的包名
# twine upload dist/* # 上传到pypi
# pip install auto_anno_2 -U -i https://pypi.org/simple # 从pypi安装
# pip list|grep auto # 查看安装的包
# pip uninstall auto_anno_2 # 卸载包
